' Snippets are unassociated sets of source.
' Must call pbsSnippetNew( "myName" ) at least once to create a snippet to work with
' pbSnippetNew automatically activates the new snippet
' Can create many different named snippets.
' Snippet names must be unique
' Most routines work on the active snippet.
' Call pbsSnippetActivate( "mySnippet" ) to activate a snippet

' Most declare routine assume a Scan has been done

Sub pbsSnippetNew Alias "pbsSnippetNew"( snippetName As String ) Export
	Local obj As iSourceFile

	If Not init_ Then Call Initialize()
	If snippets_.Contains( snippetName ) Then Exit Sub
	Call NewSourceFile( obj, "", 0 )
	activeSnippetIndex_ = snippets_.Add( snippetName, obj )
	snippet_ = obj
End Sub

Sub pbsSnippetActivate Alias "pbsSnippetActivate"( snippetName As String ) Export
	If Not init_ Then Call Initialize()
	If snippets_.Contains( snippetName ) Then
		snippet_ = snippets_.Item( snippetName )
		activeSnippetIndex_ = snippets_.Index( snippetName )
	End If
End Sub

Function pbsSnippetGetActiveName Alias "pbsSnippetGetActiveName"() Export As String
	If Not init_ Then Call Initialize()
	Function = snippets_.Key( activeSnippetIndex_ )
End Function

Sub pbsSnippetClear Alias "pbsSnippetClear"() Export
	If Not init_ Then Call Initialize()
	snippets_.Clear()
	activeSnippetIndex_ = -1
	snippet_ = Nothing
End Sub

Function pbsSnippetGetIndex Alias "pbsSnippetGetIndex"() Export As Long
	If Not init_ Then Call Initialize()
	Function = activeSnippetIndex_
End Function

Sub pbsSnippetSetIndex Alias "pbsSnippetSetIndex"( ByVal index As Long ) Export
	If Not init_ Then Call Initialize()
	If index<-1 Or index >= snippets_.Count Then Exit Sub
	If index<0 Then
		activeSnippetIndex_ = -1
		snippet_ = Nothing
	Else
		activeSnippetIndex_ = index
		snippet_ = snippets_.Value( index )
	End If
End Sub

Sub pbsSnippetClose Alias "pbsSnippetClose"( snippetName As String ) Export
	Local i As Long
	If Not init_ Then Call Initialize()
	If snippets_.Count=0 Then Exit Sub
	i = sfFileClose( snippets_, snippetName )
	If i >= activeSnippetIndex_ Then
		If activeSnippetIndex_ >= snippets_.Count Then Decr activeSnippetIndex_
		pbsSnippetSetIndex( activeSnippetIndex_ )
	ElseIf i>-1 Then
		Decr activeSnippetIndex_
		pbsSnippetSetIndex( activeSnippetIndex_ )
	End If
End Sub

Function pbsSnippetCount Alias "pbsSnippetCount"() Export As Long
	If Not init_ Then Call Initialize()
	Function = snippets_.Count
End Function

Function pbsSnippetNames Alias "pbsSnippetNames"() Export As String
	Local i As Long
	Local sb As IStringBuilderA

	If Not init_ Then Call Initialize()
	sb = Class "StringBuilderA"
	For i=0 To snippets_.Count-1
		If i Then sb.Add( $CrLf )
		sb.Add( snippets_.Key( i ) )
	Next
	Function = sb.String
End Function

Sub pbsSnippetLoad Alias "pbsSnippetLoad"( filename As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.Filename = filename
	snippet_.Reload()
End Sub

Sub pbsSnippetReload Alias "pbsSnippetReload"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.Reload()
End Sub

Sub pbsSnippetSave Alias "pbsSnippetSave"( Optional filename As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	If IsMissing( filename ) Then
		snippet_.Save()
	Else
		snippet_.Save( filename )
	End If
End Sub

Sub pbsSnippetSetText Alias "pbsSnippetSetText"( value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.Text = value
End Sub

Function pbsSnippetGetText Alias "pbsSnippetGetText"() Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = snippet_.Text
End Function

Function pbsSnippetSizeInBytes Alias "pbsSnippetSizeInBytes"() Export As Long
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = snippet_.SizeInBytes
End Function

Function pbsSnippetLineCount Alias "pbsSnippetLineCount"() Export As Long
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = snippet_.Count
End Function

Sub pbsSnippetSetLineCount Alias "pbsSnippetSetLineCount"( ByVal value As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.Count = value
End Sub

Function pbsSnippetGetLine Alias "pbsSnippetGetLine"( ByVal lineNumber As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = snippet_.Line( lineNumber )
End Function

Sub pbsSnippetSetLine Alias "pbsSnippetSetLine"( ByVal lineNumber As Long, value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.Line( lineNumber ) = value
End Sub

Function pbsSnippetGetLines Alias "pbsSnippetGetLines"( ByVal lineNumber As Long, cnt As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = snippet_.Lines( lineNumber, cnt )
End Function

Sub pbsSnippetAddText Alias "pbsSnippetAddText"( value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.AddText( value )
End Sub

Sub pbsSnippetInsertText Alias "pbsSnippetInsertText"( ByVal lineNumber As Long, value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.InsertText( lineNumber, value )
End Sub

Sub pbsSnippetDeleteText Alias "pbsSnippetDeleteText"( ByVal lineNumber As Long, cnt As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.DeleteText( lineNumber, cnt )
End Sub

Sub pbsSnippetReplaceText Alias "pbsSnippetReplaceText"( ByVal lineNumber As Long, ByVal cnt As Long, value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.ReplaceText( lineNumber, cnt, value )
End Sub

Sub pbsSnippetInsertChars Alias "pbsSnippetInsertChars"( ByVal lineNumber As Long, ByVal index As Long, insertString As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.InsertChars( lineNumber, index, insertString )
End Sub

Sub pbsSnippetDeleteChars Alias "pbsSnippetDeleteChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.DeleteChars( lineNumber, index, cnt )
End Sub

Sub pbsSnippetReplaceChars Alias "pbsSnippetReplaceChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long, insertString As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.ReplaceChars( lineNumber, index, cnt, insertString )
End Sub

Function pbsSnippetGetFilename Alias "pbsSnippetGetFilename"() Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = snippet_.Filename
End Function

Sub pbsSnippetSetFilename Alias "pbsSnippetSetFilename"( value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippet_.Filename = value
End Sub

Sub pbsSnippetRemoveContinuation Alias "pbsSnippetRemoveContinuation"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	sfRemoveContinuation( snippet_ )
End Sub

Function pbsSnippetToBBCode Alias "pbsSnippetToBBCode"( title As String, ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = sfToBBCode( snippet_, title, tabSize, keywordCasing, singleCaseIdentifier, includeMonoFormat, includeColorFormat )
End Function

Function pbsSnippetToHtml Alias "pbsSnippetToHtml"( title As String, ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = sfToHtml( snippet_, title, tabSize, keywordCasing, singleCaseIdentifier, includeMonoFormat, includeColorFormat )
End Function

Sub pbsSnippetToCasedText Alias "pbsSnippetToCasedText"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	Call sfToCasedText( snippet_, tabSize, keywordCasing, singleCaseIdentifier )
End Sub

Sub pbsSnippetReplaceToken Alias "pbsSnippetReplaceToken"( originalToken As String, newToken As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	sfReplaceToken( snippet_, originalToken, newToken )
End Sub

Sub pbsSnippetIndent Alias "pbsSnippetIndent"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	Call indenter_.Indent( snippet_ )
End Sub

Sub pbsSnippetExpandTabs Alias "pbsSnippetExpandTabs"( Optional ByVal tabSize As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	If tabSize < 1 Then tabSize = tabSize_
	If tabSize < 1 Then Exit Sub
	sfExpandTabs( snippet_, tabSize )
End Sub

Sub pbsSnippetTrimWhitespace Alias "pbsSnippetTrimWhitespace"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	Call sfTrimWhitespace( snippet_ )
End Sub

'========================================================================
' Declares
Function pbsSnippetDeclareGetInterfaces Alias "pbsSnippetDeclareGetInterfaces"() Export As String
	If Not init_ Then Call Initialize()
	Function = sfDeclareGetInterfaces( snippetDeclares_ )
End Function

Sub pbsSnippetDeclareScan Alias "pbsSnippetDeclareScan"( Optional ByVal pClear As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub
	snippetDeclares_.Scan( snippet_, activeSnippetIndex_, pClear )
End Sub

Function pbsSnippetDeclareGetName Alias "pbsSnippetDeclareGetName"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.Name( index )
End Function

Function pbsSnippetDeclareGetCount Alias "pbsSnippetDeclareGetCount"() Export As Long
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.Count()
End Function

Sub pbsSnippetDeclareClear Alias "pbsSnippetDeclareClear"() Export
	If Not init_ Then Call Initialize()
	snippetDeclares_.Clear()
End Sub

Sub pbsSnippetDeclareSetSnippet Alias "pbsSnippetDeclareSetSnippet"( snippetName As String ) Export
	Local sf As iSourceFile
	If Not init_ Then Call Initialize()
	sf = snippets_.Item( snippetName )
	If IsFalse IsInterface( sf, iSourceFile ) Then Exit Sub
	snippetDeclares_.SourceFile = sf
	snippetDeclares_.Filenumber = snippets_.Index( snippetName )
End Sub

Function pbsSnippetDeclareGetFileNumber Alias "pbsSnippetDeclareGetFileNumber"() Export As Long
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.FileNumber
End Function

'Function pbsSnippetDeclareGetName() Export As String
'  If Not init_ Then Call Initialize()
'  Function = snippets_.Key( snippetDeclares_.FileNumber )
'End Function

Sub pbsSnippetDeclareItem Alias "pbsSnippetDeclareItem"( ByVal index As Long, value As TDecl ) Export
	If Not init_ Then Call Initialize()
	value = snippetDeclares_.Item( index )
End Sub

Sub pbsSnippetDeclareGetItem Alias "pbsSnippetDeclareGetItem"( ByVal index As Long, value As TDecl ) Export
	If Not init_ Then Call Initialize()
	value = snippetDeclares_.Item( index )
End Sub

Function pbsSnippetDeclareGetDeclarationName Alias "pbsSnippetDeclareGetDeclarationName"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = GetDeclarationTypeDescription( index )
End Function

Function pbsSnippetDeclareGetClassMethods Alias "pbsSnippetDeclareGetClassMethods"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetClassMethods( index )
End Function

Function pbsSnippetDeclareGetClassOutline Alias "pbsSnippetDeclareGetClassOutline"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetClassOutline( index )
End Function

Function pbsSnippetDeclareGetInstanceVariables Alias "pbsSnippetDeclareGetInstanceVariables"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetInstanceVariables( index )
End Function

Function pbsSnippetDeclareGetClassInterfaces Alias "pbsSnippetDeclareGetClassInterfaces"( ByVal index As Long) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetClassInterfaces( index )
End Function

Function pbsSnippetDeclareGetSignature Alias "pbsSnippetDeclareGetSignature"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetSignature( index )
End Function

Function pbsSnippetDeclareGetDefinition Alias "pbsSnippetDeclareGetDefinition"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetDefinition( index )
End Function

Sub pbsSnippetStandardizeExportAliases Alias "pbsSnippetStandardizeExportAliases"( exportList As String ) Export
	If Not init_ Then Call Initialize()
	snippetDeclares_.StandardizeExportAliases( exportList )
End Sub

Sub pbsSnippetAddDebugAliases Alias "pbsSnippetAddDebugAliases"() Export
	If Not init_ Then Call Initialize()
	snippetDeclares_.AddDebugAliases()
End Sub

Sub pbsSnippetRemoveAliases Alias "pbsSnippetRemoveAliases"() Export
	If Not init_ Then Call Initialize()
	snippetDeclares_.RemoveAliases()
End Sub

Function pbsSnippetDeclareGetNames Alias "pbsSnippetDeclareGetNames"( ByVal declType As Long ) Export As String
	Local u As iStringBag
	Local l(), v As String
	Local i, c As Long
	Local sb As IStringBuilderA

	If Not init_ Then Call Initialize()
	v = snippetDeclares_.GetNames( declType )
	c = StrSplit( v, l(), $CrLf )
	Call NewStringBag( u, c, c )
	sb = Class "StringBuilderA"
	For i=0 To c-1
		If l(i)<>"" And Not u.Contains( l(i) ) Then
			u.Add( l(i) )
			If sb.Len Then sb.Add( $CrLf )
			sb.Add( l(i) )
		End If
	Next
	Function = sb.String
End Function

Function pbsSnippetGetInterfaceRoutines Alias "pbsSnippetGetInterfaceRoutines"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetInterfaceRoutines( index )
End Function

Function pbsSnippetGetInterfaceStub Alias "pbsSnippetGetInterfaceStub"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetInterfaceStub( index )
End Function

Function pbsSnippetGetInterfaceClasses Alias "pbsSnippetGetInterfaceClasses"( interfaceName As String ) Export As String
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.GetInterfaceClasses( interfaceName )
End Function

Function pbsSnippetDeclareFindName Alias "pbsSnippetDeclareFindName"( ByVal start As Long, value As String, ByVal declType As Long ) Export As Long
	If Not init_ Then Call Initialize()
	Function = snippetDeclares_.FindName( start, value, declType )
End Function

Function pbsSnippetBuildDllInclude Alias "pbsSnippetBuildDllInclude"( libName As String ) Export As String
	Local exportList, routines, interfaces, constants, enums, types, unions As String

	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function

	exportList = sfFindExports( snippet_ )
	Call pbsSnippetDeclareScan( -1 )
	Call snippetDeclares_.GetDLLDeclaration( libName, exportList, routines, interfaces, constants, enums, types, unions )
	Function = BuildInc( "Include file for users of " + libName, routines, interfaces, constants, enums, types, unions )
End Function

Function pbsSnippetBuildSllInclude Alias "pbsSnippetBuildSllInclude"( libName As String ) Export As String
	Local exportList, routines, interfaces, constants, enums, types, unions As String

	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function

	exportList = sfFindExports( snippet_ )
	Call pbsSnippetDeclareScan( -1 )
	Call snippetDeclares_.GetSLLDeclaration( exportList, routines, interfaces, constants, enums, types, unions )
	Function = BuildInc( "Include file for users of " + libName, routines, interfaces, constants, enums, types, unions )
End Function

Function pbsSnippetGetInterfaceDeclaration Alias "pbsSnippetGetInterfaceDeclaration"( ByVal index As Long, constants As String ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function

	Function = snippetDeclares_.GetInterfaceDeclaration( index, constants )
End Function
'============================================================================
Function pbsSnippetFindExports Alias "pbsSnippetFindExports"() Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Function
	Function = sfFindExports( snippet_ )
End Function

Sub pbsSnippetManageBlankLines Alias "pbsSnippetManageBlankLines"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub

	sfManageBlankLines( snippet_ )
End Sub

Sub pbsSnippetInsertClass Alias "pbsSnippetInsertClass"( ByVal lineNumber As Long, className As String, Optional ByVal addGUID As Long, Optional ByVal isCommon As Long ) Export
	Local g As String

	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub

	If addGUID Then g = " Guid$(" + Wrap$( LCase$( GuidTxt$( Guid$ ) ), $Dq, $Dq ) + ")"
	If isCommon Then g += " Common"
	snippet_.InsertText( lineNumber, Build$( "Class ", className, g, $CrLf, "End Class" ) )
End Sub

Sub pbsSnippetInsertInterface Alias "pbsSnippetInsertInterface"( ByVal lineNumber As Long, interfaceName As String, inheritFrom As String, Optional ByVal addGUID As Long ) Export
	Local g As String

	If Not init_ Then Call Initialize()
	If IsNothing( snippet_ ) Then Exit Sub

	If addGUID Then g = " Guid$(" + Wrap$( LCase$( GuidTxt$( Guid$ ) ), $Dq, $Dq ) + ")"
	snippet_.InsertText( lineNumber, Build$( "  Interface ", interfaceName, g, $CrLf, "    Inherit ", inheritFrom, $CrLf, "  End Interface" ) )
End Sub
