Sub NewLongBag( obj As iLongBag, ByVal hashSize As Long, Optional resizeAmount As Long )
	obj = Class "cLongBag"
	obj.HashCount = hashSize
	If Not IsMissing( resizeAmount ) Then
		obj.ResizeAmount = resizeAmount
	End If
End Sub

Class cLongBag
	Instance hash_() As Long
	Instance next_() As Long
	Instance key_() As Long
	Instance hashCount_ As Long
	Instance dataCount_ As Long
	Instance ubData_ As Long
	Instance dataResizeAmount_ As Long
	Debug_ClassInstance

	Class Method Create()
		dataResizeAmount_ = 100
		Debug_ClassCreate("cLongBag")
	End Method
	Debug_DefineClassDestroy("cLongBag")

	Interface iLongBag
		Inherit IUnknown

		Property Get HashCount() As Long
			Property = hashCount_
		End Property
		Property Set HashCount( value As Long )
			Me.ResizeHash( value )
		End Property

		Property Get ResizeAmount() As Long
			Property = dataResizeAmount_
		End Property
		Property Set ResizeAmount( ByVal value As Long )
			If value>0 Then dataResizeAmount_ = value
		End Property

		Property Get Count() As Long
			Property = dataCount_
		End Property
		Property Set Count( ByVal value As Long )
			If value> dataCount_ Then Me.ResizeData( value )
		End Property

#If %Def( %DEBUGGING )
		Method GetStats( colisions As Long, maxColisionLength As Long, avgColisionLength As Double )
			Local i As Long
			Local c, l, ml, tl As Long
			Local index As Long

			For i=0 To hashCount_-1
				index = hash_( i )
				If index >-1 Then
					If next_( index )>-1 Then
						Incr c
						l = 0
						Do
							Incr l
							index = next_( index )
						Loop While index>-1
						If l>ml Then ml=l
						tl += l
					End If
				End If
			Next
			colisions = c
			maxColisionLength = ml
			avgColisionLength = CDbl(tl) / CDbl(c)
		End Method
#EndIf

		Method Clear()
			Me.ResizeHash( 0 )
			Me.ResizeData( 0 )
		End Method

		' Returns index of added item
		Method Add( pKey As Long ) As Long
			Register index As Long
			Register lastIndex As Long
			Local hc As Long
			Local np As Long Ptr

			' Notes: Long returns from functions are OK.
			' Long returns better passed to sub by reference (like 3 times slower from function)
			IntHash(pKey,hashCount_,hc): ' Would be faster inline, but instance variable access from ASM I'm not sure I understand yet.

			index = hash_( hc )

			If index = -1 Then
				' First entry
				If dataCount_>=ubData_ Then Me.ResizeData( ubData_+dataResizeAmount_ )
				hash_( hc ) = dataCount_
			Else
				' Collision
				np = VarPtr( next_(0) )
				Do
					If pKey = key_( index ) Then
						lastIndex = index
						index = @np[ index ]
					Else
						Method = index
						Exit Method
					End If
				Loop While index>-1
				If dataCount_>=ubData_ Then Me.ResizeData( ubData_+dataResizeAmount_ )
				next_( lastIndex ) = dataCount_
			End If
			next_( dataCount_ ) = -1
			key_( dataCount_ ) = pKey
			Method = dataCount_
			Incr dataCount_
		End Method

		' -1 if found, 0 if not found
		Property Get Contains( pKey As Long ) As Long
			Property = ( Me.Index( pKey ) > -1 )
		End Property

		Property Get Index( pKey As Long ) As Long
			' Returns index of key
			' Returns -1 if not found
			' Returns 0 - dataCount_-1 if found
			Register index As Long
			Local hc As Long

			IntHash( pKey, hashCount_, hc )
			index = hash_( hc )

			While index > -1
				If pKey = key_( index ) Then
					index = next_( index )
				Else
					Property = index
					Exit Property
				End If
			Wend
			Property = -1
		End Property

		Property Get Key( ByVal index As Long ) As Long
			If index>=0 And index< dataCount_ Then
				Property = key_( index )
			End If
		End Property

		Property Get Item( pKey As Long ) As Long
			Local index As Long
			index = Me.Index( pKey )
			If index>-1 Then Property = key_( index )
		End Property
	End Interface

	Class Method ResizeData( ByVal value As Long )
		If value>0 Then
			If value < dataCount_ Then
				' Have to clear hash if shrinking, otherwise there will be invalid entries
				If hashCount_>0 Then Memory Fill VarPtr( hash_(0) ), value*SizeOf(value), Byte &HFF
				dataCount_ = 0
			End If
			ReDim Preserve next_( value-1 )
			ReDim Preserve key_( value-1 )
			ubData_ = value
		Else
			Erase next_()
			Erase key_()
			ubData_ = 0
			dataCount_ = 0
			' Have to clear hash if shrinking, otherwise there will be invalid entries
			If hashCount_>0 Then Memory Fill VarPtr( hash_(0) ), value*SizeOf(value), Byte &HFF
		End If
	End Method

	Class Method ResizeHash( ByVal value As Long )
		' Clears bag
		value = CollectionSize( value ): ' Make a prime >= value
		ReDim hash_( value-1 )
		Memory Fill VarPtr( hash_(0) ), value*SizeOf(value), Byte &HFF
		dataCount_ = 0
		hashCount_ = value
	End Method
End Class
