' These lists identify where words occur either as declarations or as references
' Note that it's all declarations and references, not just related ones.
' i.e. the declaration i might occur in many routines with many references in each
' Can spend time matching them up if someone cares, just keeping us from scanning
' the entire declaration list to find all instances of a word, also gives us an idea of symbol usage.
'
' Today COM sucks.  Wanted to use AddDecl, AddRef, no love AddRef taken, what are the odds? :)
'
' I hate multi-purposing a class but having a string key value able to return either declaration or references seemed
' good at the time.  The alternative would be to create two classes but then they wouldn't be synchronized.

Type tDeclList
	' This type is internal and should is not needed by users
	decl As Dword: ' Handle to memory block of declaration indexes
	declCount As Long: ' Number of declarations in memory block
	ref As Dword: ' Handle to memory block of reference indexes
	refCount As Long: ' # of references in memory block
End Type

Sub NewStringDeclMap( obj As iStringDeclMap, ByVal hashSize As Long, Optional resizeAmount As Long )
	' Passing the object to create by reference is 1) slightly faster than using a function, and 2) results in correct ref counts (functions leave an extra ref count in 10.03)
	obj = Class "cStringDeclMap"
	obj.HashCount = hashSize
	If Not IsMissing( resizeAmount ) Then
		obj.ResizeAmount = resizeAmount
	End If
End Sub

Class cStringDeclMap
	Instance hash_() As Long
	Instance next_() As Long
	Instance key_() As String
	Instance value_() As tDeclList
	Instance hashCount_ As Long
	Instance dataCount_ As Long
	Instance ubData_ As Long
	Instance dataResizeAmount_ As Long
	Debug_ClassInstance

	Class Method Create()
		dataResizeAmount_ = 100
		Debug_ClassCreate("cStringDeclMap")
	End Method
	Debug_DefineClassDestroy("cStringDeclMap")

	Interface iStringDeclMap
		Inherit IUnknown

		Property Get HashCount() As Long
			Property = hashCount_
		End Property
		Property Set HashCount( value As Long )
			Me.ResizeHash( value )
		End Property

		Property Get ResizeAmount() As Long
			Property = dataResizeAmount_
		End Property
		Property Set ResizeAmount( ByVal value As Long )
			If value>0 Then dataResizeAmount_ = value
		End Property

		Property Get Count() As Long
			Property = dataCount_
		End Property
		Property Set Count( ByVal value As Long )
			If value> dataCount_ Then Me.ResizeData( value )
		End Property

		Method Clear()
			Me.ResizeHash( 0 )
			Me.ResizeData( 0 )
		End Method

		' Returns index of added item
		Method AddDeclaration( pKey As String, value As Long ) As Long
			#Debug Code Off
			Register index As Long
			Register lastIndex As Long
			Local hc As Long
			Local np As Long Ptr

			' Notes: Long returns from functions are OK.
			' String returns better passed to sub by reference (like 3 times slower from function)
			hc = StringHash( pKey, hashCount_ ): ' Would be faster inline, but instance variable access from ASM I'm not sure I understand yet.

			index = hash_( hc )

			If index = -1 Then
				' First entry
				If dataCount_>=ubData_-1 Then Me.ResizeData( ubData_+dataResizeAmount_ )
				hash_( hc ) = dataCount_
			Else
				' Collision
				np = VarPtr( next_(0) )
				Do
					If StrStrICmp( pKey, key_( index ) ) Then
						lastIndex = index
						index = @np[ index ]
						'index = next_( index )
					Else
						Method = index
						Me.iAddDecl( index, value )
						Exit Method
					End If
				Loop While index>-1
				If dataCount_>=ubData_-1 Then Me.ResizeData( ubData_+dataResizeAmount_ )
				next_( lastIndex ) = dataCount_
			End If
			next_( dataCount_ ) = -1
			key_( dataCount_ ) = pKey
			Method = dataCount_
			Incr dataCount_
			Me.iAddDecl( dataCount_-1, value )
			#Debug Code On
		End Method

		Method AddReference( pKey As String, value As Long ) As Long
			Register index As Long
			Register lastIndex As Long
			Local hc As Long
			Local np As Long Ptr

			' Notes: Long returns from functions are OK.
			' String returns better passed to sub by reference (like 3 times slower from function)
			hc = StringHash( pKey, hashCount_ ): ' Would be faster inline, but instance variable access from ASM I'm not sure I understand yet.

			index = hash_( hc )

			If index = -1 Then
				' First entry
				If dataCount_>=ubData_-1 Then Me.ResizeData( ubData_+dataResizeAmount_ )
				hash_( hc ) = dataCount_
			Else
				' Collision
				np = VarPtr( next_(0) )
				Do
					If StrStrICmp( pKey, key_( index ) ) Then
						lastIndex = index
						index = @np[ index ]
						'index = next_( index )
					Else
						Method = index
						Me.iAddRef( index, value )
						Exit Method
					End If
				Loop While index>-1
				If dataCount_>=ubData_-1 Then Me.ResizeData( ubData_+dataResizeAmount_ )
				next_( lastIndex ) = dataCount_
			End If
			next_( dataCount_ ) = -1
			key_( dataCount_ ) = pKey
			Method = dataCount_
			Incr dataCount_
			Me.iAddRef( dataCount_-1, value )
		End Method

		' -1 if found, 0 if not found
		Property Get Contains( pKey As String ) As Long
			' Cost .04 second for a million lookups to use the function instead of inlining it
			Property = ( Me.Index( pKey ) > -1 )
		End Property

		Property Get Index( pKey As String ) As Long
			#Debug Code Off
			' Returns index of key
			' Returns -1 if not found
			' Returns 0 - dataCount_-1 if found
			Register index As Long
			Local hc As Long

			hc = StringHash( pKey, hashCount_ )
			index = hash_( hc )

			While index > -1
				If StrStrICmp( pKey, key_( index ) ) Then
					index = next_( index )
				Else
					Property = index
					Exit Property
				End If
			Wend
			Property = -1
			#Debug Code On
		End Property

		Property Get Key( ByVal index As Long ) As String
			If index>=0 And index< dataCount_ Then
				Property = key_( index )
			End If
		End Property

		Property Get DeclarationCount( ByVal index As Long ) As Long
			If index<0 Or index>=dataCount_ Then Exit Property
			Property = value_( index ).declCount
		End Property
		Property Get ReferenceCount( ByVal index As Long ) As Long
			If index<0 Or index>=dataCount_ Then Exit Property
			Property = value_( index ).refCount
		End Property


		Property Get Declaration( ByVal index As Long, ByVal declIndex As Long ) As Long
			Local pMem As Long Ptr
			Local isLocked As Long

			If index<0 Or index>=dataCount_ Then Exit Property
			If declIndex<0 Or declIndex>= value_( index ).declCount Then Exit Property
			GlobalMem Lock value_( index ).decl To pMem
			Property = @pMem[ declIndex ]
			GlobalMem UnLock value_( index ).decl To isLocked
		End Property

		Property Get Reference( ByVal index As Long, ByVal refIndex As Long ) As Long
			Local pMem As Long Ptr
			Local isLocked As Long

			If index<0 Or index>=dataCount_ Then Exit Property
			If refIndex<0 Or refIndex>= value_( index ).refCount Then Exit Property
			GlobalMem Lock value_( index ).ref To pMem
			Property = @pMem[ refIndex ]
			GlobalMem UnLock value_( index ).ref To isLocked
		End Property
	End Interface

	Class Method ResizeData( ByVal value As Long )
		If value>0 Then
			ReDim Preserve next_( value-1 )
			ReDim Preserve key_( value-1 )
			ReDim Preserve value_( value-1 )
			ubData_ = value
			dataCount_ = Min&( ubData_-1, dataCount_ )
		Else
			Erase next_()
			Erase key_()
			Erase value_()
			ubData_ = 0
			dataCount_ = 0
		End If
	End Method

	Class Method ResizeHash( ByVal value As Long )
		value = CollectionSize( value ): ' Make a prime >= value
		ReDim hash_( value-1 )
		Memory Fill VarPtr( hash_(0) ), value*SizeOf(value), Byte &HFF
		hashCount_ = value
	End Method

	Class Method iAddDecl( ByVal index As Long, ByVal value As Long )
		#Debug Code Off
		Local sz, nsz As Long
		Local hMem, hFree, isLocked As Long
		Local pSrc, pDst As Long Ptr

		If value_( index ).decl = 0 Then
			' Initial allocation
			GlobalMem Alloc 4 * SizeOf( index ) To value_( index ).decl
		Else
			' Need more room, allocate, copy, and free
			GlobalMem Size value_( index ).decl To sz
			If value_( index ).declCount >= sz\4 Then
				nsz = sz+ (5*SizeOf(value))
				GlobalMem Alloc nsz To hMem
				GlobalMem Lock value_( index ).decl To pSrc
				GlobalMem Lock hMem To pDst
				Memory Copy pSrc, pDst, sz
				GlobalMem UnLock hMem To isLocked
				GlobalMem UnLock value_( index ).decl To isLocked
				GlobalMem Free value_( index ).decl To hFree
				value_( index ).decl = hMem
			End If
		End If
		GlobalMem Lock value_( index ).decl To pDst
		@pDst[ value_( index ).declCount ] = value
		Incr value_( index ).declCount
		GlobalMem UnLock value_( index ).decl To isLocked
		#Debug Code Off
	End Method

	Class Method iAddRef( ByVal index As Long, ByVal value As Long )
		Local sz, nsz As Long
		Local hMem, hFree, isLocked As Long
		Local pSrc, pDst As Long Ptr

		If index<0 Or index>dataCount_ Then Exit Method

		If value_( index ).ref = 0 Then
			' Initial allocation
			GlobalMem Alloc 4 * SizeOf( index ) To value_( index ).ref
		Else
			' Need more room, allocate, copy, and free
			GlobalMem Size value_( index ).ref To sz
			If value_( index ).refCount >= sz\4 Then
				nsz = sz+ (5*SizeOf(value))
				GlobalMem Alloc nsz To hMem
				GlobalMem Lock value_( index ).ref To pSrc
				GlobalMem Lock hMem To pDst
				Memory Copy pSrc, pDst, sz
				GlobalMem UnLock hMem To isLocked
				GlobalMem UnLock value_( index ).ref To isLocked
				GlobalMem Free value_( index ).ref To hFree
				value_( index ).ref = hMem
			End If
		End If
		GlobalMem Lock hMem To pDst
		@pDst[ value_( index ).refCount ] = value
		Incr value_( index ).refCount
		GlobalMem UnLock hMem To isLocked
	End Method

	Class Method ReleaseMem()
		Local i As Long
		Local hFree As Dword

		For i= 0 To dataCount_ - 1
			If value_( i ).ref Then
				GlobalMem Free value_( i ).ref To hFree
				value_( i ).ref = 0
				value_( i ).refCount = 0
			End If
			If value_( i ).decl Then
				GlobalMem Free value_( i ).decl To hFree
				value_( i ).decl = 0
				value_( i ).declCount = 0
			End If
		Next
	End Method
End Class
