Sub NewStringSourcefileMap( obj As iStringSourcefileMap, ByVal hashSize As Long, Optional resizeAmount As Long )
	' Passing the object to create by reference is 1) slightly faster than using a function, and 2) results in correct ref counts (functions leave an extra ref count in 10.03)
	obj = Nothing
	obj = Class "cStringSourcefileMap"
	obj.HashCount = hashSize
	If Not IsMissing( resizeAmount ) Then
		obj.ResizeAmount = resizeAmount
	End If
End Sub

Sub DupStringSourcefileMap( obj As iStringSourcefileMap, src As iStringSourcefileMap, Optional ByVal useKeys As Long )
	Local i As Long
	Local value As String
	Local sf As iSourceFile
	obj = Nothing
	If IsFalse IsInterface( src, iStringSourcefileMap ) Then Exit Sub
	obj = Class "cStringSourcefileMap"
	obj.HashCount = src.HashCount
	obj.ResizeAmount = src.ResizeAmount
	For i=0 To src.Count-1
		sf = src.Value(i)
		value = IIf$( useKeys, src.Key( i ), sf.Filename )
		obj.Add( value, sf )
		sf = Nothing
	Next
End Sub

Class cStringSourcefileMap
	Instance hash_() As Long
	Instance next_() As Long
	Instance key_() As String
	Instance value_() As iSourceFile
	Instance hashCount_ As Long
	Instance dataCount_ As Long
	Instance ubData_ As Long
	Instance dataResizeAmount_ As Long
	Debug_ClassInstance

	Class Method Create()
		dataResizeAmount_ = 100
		Debug_ClassCreate("cStringSourcefileMap")
	End Method
	Debug_DefineClassDestroy("cStringSourcefileMap")

	Interface iStringSourcefileMap
		Inherit IUnknown

		Property Get HashCount() As Long
			Property = hashCount_
		End Property
		Property Set HashCount( value As Long )
			Me.ResizeHash( value )
		End Property

		Property Get ResizeAmount() As Long
			Property = dataResizeAmount_
		End Property
		Property Set ResizeAmount( ByVal value As Long )
			If value>0 Then dataResizeAmount_ = value
		End Property

		Property Get Count() As Long
			Property = dataCount_
		End Property
		Property Set Count( ByVal value As Long )
			If value> dataCount_ Then Me.ResizeData( value )
		End Property

		Method Clear()
			Me.ResizeHash( 0 )
			Me.ResizeData( 0 )
		End Method

		' Returns index of added item
		Method Add( pKey As String, value As iSourceFile ) As Long
			Register index As Long
			Register lastIndex As Long
			Local hc As Long
			Local np As Long Ptr

			' Notes: Long returns from functions are OK.
			' String returns better passed to sub by reference (like 3 times slower from function)
			hc = StringHash( pKey, hashCount_ ): ' Would be faster inline, but instance variable access from ASM I'm not sure I understand yet.

			index = hash_( hc )

			If index = -1 Then
				' First entry
				If dataCount_>=ubData_-1 Then Me.ResizeData( ubData_+dataResizeAmount_ )
				hash_( hc ) = dataCount_
			Else
				' Collision
				np = VarPtr( next_(0) )
				Do
					If StrStrICmp( pKey, key_( index ) ) Then
						lastIndex = index
						index = @np[ index ]
						'index = next_( index )
					Else
						Method = index
						Exit Method
					End If
				Loop While index>-1
				If dataCount_>=ubData_-1 Then Me.ResizeData( ubData_+dataResizeAmount_ )
				next_( lastIndex ) = dataCount_
			End If
			next_( dataCount_ ) = -1
			key_( dataCount_ ) = pKey
			value_( dataCount_ ) = value
			Method = dataCount_
			Incr dataCount_
		End Method

		' -1 if found, 0 if not found
		Property Get Contains( pKey As String ) As Long
			' Cost .04 second for a million lookups to use the function instead of inlining it
			Property = ( Me.Index( pKey ) > -1 )
		End Property

		Property Get Index( pKey As String ) As Long
			' Returns index of key
			' Returns -1 if not found
			' Returns 0 - dataCount_-1 if found
			Register index As Long
			Local hc As Long

			hc = StringHash( pKey, hashCount_ )
			index = hash_( hc )

			While index > -1
				If StrStrICmp( pKey, key_( index ) ) Then
					index = next_( index )
				Else
					Property = index
					Exit Property
				End If
			Wend
			Property = -1
		End Property

		Property Get Key( ByVal index As Long ) As String
			If index>=0 And index< dataCount_ Then
				Property = key_( index )
			End If
		End Property

		Property Get Value( ByVal index As Long ) As iSourceFile
			If index>=0 And index<dataCount_ Then
				Property = value_( index )
			End If
		End Property
		Property Set Value( ByVal index As Long, sf As iSourceFile )
			If index>=0 And index<dataCount_ Then
				value_( index ) = Nothing: ' Ensure ref count ok (PB 10.03)
				value_( index ) = sf
			End If
		End Property

		Property Get Item( pKey As String ) As iSourceFile
			Local index As Long
			index = Me.Index( pKey )
			If index>-1 Then Property = value_( index )
		End Property
		Property Set Item( pKey As String, value As iSourceFile )
			Local index As Long
			index = Me.Index( pKey )
			If index>-1 Then
				value_( index ) = value
			End If
		End Property
	End Interface

	Class Method ResizeData( ByVal value As Long )
		If value>0 Then
			ReDim Preserve next_( value-1 )
			ReDim Preserve key_( value-1 )
			ReDim Preserve value_( value-1 )
			ubData_ = value
			dataCount_ = Min&( ubData_-1, dataCount_ )
		Else
			Erase next_()
			Erase key_()
			Erase value_()
			ubData_ = 0
			dataCount_ = 0
		End If
	End Method

	Class Method ResizeHash( ByVal value As Long )
		value = CollectionSize( value ): ' Make a prime >= value
		ReDim hash_( value-1 )
		Memory Fill VarPtr( hash_(0) ), value*SizeOf(value), Byte &HFF
		hashCount_ = value
	End Method
End Class
