' Note token pointer values returned from LineTokenizer are temporary and become
' invalid with the next SetLine is called or when object is destroyed.
Sub NewLineTokenizer( obj As iLineTokenizer, keywords As iStringBag ): ', builtin As iStringBag, Optional tabSize As Long )
	obj = Class "cLineTokenizer"
	obj.Keywords = keywords
End Sub

Class cLineTokenizer
	Instance tkn_() As String
	Instance tokenCol_() As Long
	Instance tokenType_() As Long
	Instance keywords_ As iStringBag
	Instance cnt_ As Long
	Debug_ClassInstance

	Class Method Create()
		Dim tkn_(127)
		Dim tokenCol_(127)
		Dim tokenType_(127)
		Debug_ClassCreate("cLineTokenizer")
	End Method
	Debug_DefineClassDestroy("cLineTokenizer")

	Interface iLineTokenizer
		Inherit IUnknown

		Property Get Keywords() As iStringBag
			Property = keywords_
		End Property
		Property Set Keywords( value As iStringBag )
			keywords_ = value
		End Property

		Property Get Count() As Long
			#Debug Code Off
			Property = cnt_
			#Debug Code On
		End Property

		' Allow tokenizing a random line not in a source file
		Method SetLine( text_ As String )
			#Debug Code Off
			'index_ = -1
			Me.TokenizeLine( StrPtr( text_ ), Len( text_ ) )
			#Debug Code On
		End Method

		Method SetLinePointer( textPtr As Dword )
			#Debug Code Off
			Local p As StringZ Ptr
			If textPtr = 0 Then Exit Method
			p = textPtr
			Me.TokenizeLine( textPtr, Len( @p ) )
			#Debug Code On
		End Method

		Method GetToken( ByVal i As Long, pToken As Dword, pColumn As Long, pTokenType As Long ) As Long
			Static noToken As String
			#Debug Code Off
			If i>=0 And i<cnt_ Then
				pColumn = tokenCol_( i )
				pToken = StrPtr( tkn_( i ) )
				pTokenType = tokenType_( i )
				Method = -1
			Else
				pToken = StrPtr( noToken )
				pColumn = -1
				pTokenType = %TokenTypes.eUnknown
			End If
			#Debug Code On
		End Method
	End Interface

	Class Method TokenizeLine( ByVal line_ As StringZ Ptr*2048, ByVal lineLength_ As Long )
		'#Debug Code Off
		Register curChar_ As Long
		Register ll_ As Long
		Local tokenType As Long
		Local oddQuote As Long
		Local ln As Long
		Local fb As Byte
		Local p As Byte Ptr
		Local p2 As Byte Ptr
		Local tkn As String
		Local tabSize_ As Long
		Local buf As StringZ * 2048
		Local sz As Long
		Local expands As Long

		p = line_
		cnt_ = 0
		ll_ = lineLength_
		If line_=0 Or lineLength_=0 Then
			Exit Method
		End If

		While curChar_ < ll_
			tokenType = %TokenTypes.eUnknown
			' Skip whitepace (space, tab), will be 0 at end of line
			While @p[curChar_] = 32 Or @p[curChar_] = 9
				Incr curChar_
			Wend
			If curChar_ >= ll_ Then Exit Method
			tokenCol_(cnt_) = curChar_
			Select Case Const @p[curChar_]
				Case 43, 45, 42, 47, 92  ' +-*/\
					tokenType = %TokenTypes.eOperator
					Incr curChar_
					If @p[curChar_] = 61 Then Incr curChar_ ' =
				Case 33: ' ! Assembly
					curChar_ = ll_
					tokenType = %TokenTypes.eAsm
				Case 94, 64 ' ^, @ no ^= ... seems odd
					tokenType = %TokenTypes.eOperator
					Incr curChar_
				Case 60, 62, 61  ' <>=
					tokenType = %TokenTypes.eOperator
					Incr curChar_
					Select Case Const @p[curChar_]
						Case 60, 62, 61: Incr curChar_
					End Select
				Case 39 ' ' - Comment
					curChar_ = ll_
					tokenType = %TokenTypes.eRemark
				Case 95: ' _ = Line Continuation
					Incr curChar_
					tokenType = %TokenTypes.eContinue
				Case 38 ' &
					fb = @p[curChar_]
					Incr curChar_
					If curChar_ = ll_ Then Exit Select
					Select Case Const @p[curChar_]
						Case 61 ' &=
							Incr curChar_
							tokenType = %TokenTypes.eOperator
						Case 72, 104 '&H, &h
							tokenType = %TokenTypes.eNumber
							Incr curChar_
							GoSub GetHexidecimal
							GoSub GetSuffix
						Case 66, 98 ' &B, &b
							tokenType = %TokenTypes.eNumber
							Incr curChar_
							GoSub GetBinary
							GoSub GetSuffix
						Case 79, 111 ' &O, &o
							tokenType = %TokenTypes.eNumber
							Incr curChar_
							GoSub GetOctal
							GoSub GetSuffix
					End Select
				Case 35, 36, 37 ' #$%
					' Should just be prefixes like: #include, $include, $$constant, %constant (@ is an operator)
					fb = @p[curChar_]
					Incr curChar_
					If fb = 36 And @p[curChar_] = 36 Then Incr curChar_: ' $
					GoSub GetIdentifier
					tokenType = %TokenTypes.eIdentifier
					' Directives will get set to builtin
				Case 34 ' "- String, have to handle embeded ""... :(
					tokenType = %TokenTypes.eString
					Incr curChar_
					oddQuote = -1
					Do
						While curChar_ < ll_ And @p[curChar_] <> 34
							Incr curChar_
						Wend
						If curChar_ < ll_ Then
							' "
							oddQuote = Not oddQuote
							Incr curChar_
						End If
					Loop While curChar_ < ll_ And (oddQuote Or @p[curChar_] = 34)
				Case 40, 41, 91, 93, 44, 46, 58, 59 '  ()[],.:;_
					Incr curChar_
					tokenType = %TokenTypes.eOperator
				Case 48, 49, 50, 51, 52, 53, 54, 55, 56, 57 ' 0-9
					GoSub GetNumber
					GoSub GetSuffix
					tokenType = %TokenTypes.eNumber
				Case Else
					GoSub GetIdentifier  ' A-Z, a-Z, 0-9, _
					GoSub GetSuffix  ' ! # % ? & $ - Type specifiers
					tokenType = %TokenTypes.eIdentifier
			End Select
			' This should never happen, if it starts to happen on valid code, tokenizer needs an update :)
			If tokenCol_(cnt_) = curChar_ Then
				Incr curChar_
			End If
			ln = curChar_ - tokenCol_(cnt_)
			If curChar_ < ll_ And @p[curChar_] = 61 And ln>1 And ln<4 Then
				' = (mod=, not=, and=, or=...)
				p2 = p + tokenCol_(cnt_)
				Select Case Const @p2
					Case 77, 109:                                          ' m
						If @p2[1]<>79 And @p2[1]<>111 Then GoTo NotOperator  ' o
						If @p2[2]<>68 And @p2[2]<>100  Then GoTo NotOperator ' d

					Case 78, 110:                                          ' n
						If @p2[1]<>79 And @p2[1]<>111  Then GoTo NotOperator ' o
						If @p2[2]<>84 And @p2[2]<>116  Then GoTo NotOperator ' t

					Case 65, 97:                                           ' a
						If @p2[1]<>78 And @p2[1]<>110  Then GoTo NotOperator ' n
						If @p2[2]<>68 And @p2[2]<>100  Then GoTo NotOperator ' d

					Case 79, 111:                                          ' o
						If @p2[1]<>82 And @p2[1]<>114  Then GoTo NotOperator ' r

					Case 88, 120:                                          ' x
						If @p2[1]<>79 And @p2[1]<>111  Then GoTo NotOperator ' o
						If @p2[2]<>82 And @p2[2]<>114  Then GoTo NotOperator ' r

					Case 69, 101:                                          ' e
						If @p2[1]<>81 And @p2[1]<>113  Then GoTo NotOperator ' q
						If @p2[2]<>86 And @p2[2]<>118  Then GoTo NotOperator ' v

					Case 73, 105:                                          ' i
						If @p2[1]<>77 And @p2[1]<>109  Then GoTo NotOperator ' m
						If @p2[2]<>80 And @p2[2]<>112  Then GoTo NotOperator ' p
					Case Else
						GoTo NotOperator
				End Select
				Incr curChar_
				Incr ln
				tokenType = %TokenTypes.eOperator
NotOperator:
			End If
			If ln<>3 Then GoTo NotRemAsm: ' Can't be Rem or Asm...
			p2 = p + tokenCol_(cnt_)
			Select Case Const @p2
				Case 82, 114:                                       ' r
					If @p2[1]<>69 And @p2[1]<>101 Then GoTo NotRemAsm ' e
					If @p2[2]<>77 And @p2[2]<>109 Then GoTo NotRemAsm ' m
					curChar_ = ll_
					ln = curChar_ - tokenCol_(cnt_)
					tokenType = %TokenTypes.eRemark

				Case 65, 97:                                       ' a
					If @p2[1]<>83 And @p[1]<>115 Then GoTo NotRemAsm ' s
					If @p2[2]<>77 And @p[2]<>109 Then GoTo NotRemAsm ' m
					curChar_ = ll_
					ln = curChar_ - tokenCol_(cnt_)
					tokenType = %TokenTypes.eAsm

				Case Else
					GoTo NotRemAsm
			End Select
NotRemAsm:
			tkn_( cnt_ ) = Peek$( p + tokenCol_(cnt_), ln )
			If tokenType=%TokenTypes.eIdentifier And keywords_.Contains( tkn_(cnt_) ) Then tokenType = %TokenTypes.eKeyword
			If tokenType=%TokenTypes.eIdentifier And (@p[tokenCol_(cnt_)]=%cDollar Or @p[tokenCol_(cnt_)]=%cPercent) Then tokenType = %TokenTypes.eConstant
			tokenType_( cnt_ ) = tokenType
			Incr cnt_
		Wend
		Exit Method

GetBinary:
		While 1
			Select Case Const @p[curChar_]
					'01
				Case 48, 49
					Incr curChar_
				Case Else
					Return
			End Select
		Wend

GetOctal:
		While 1
			Select Case Const @p[curChar_]
					'01234567
				Case 48, 49, 50, 51, 52, 53, 54, 55
					Incr curChar_
				Case Else
					Return
			End Select
		Wend

GetHexidecimal:
		While 1
			Select Case Const @p[curChar_]
					'0123456789ABCEFabcdef
				Case 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 65, 66, 67, 69, 70, 97, 98, 99, 100, 101, 102
					Incr curChar_
				Case Else
					Return
			End Select
		Wend

GetNumber:
		While 1
			Select Case Const @p[curChar_]
					'0123456789+-.dDeE
				Case 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 43, 45, 46, 100, 68, 101, 69
					Incr curChar_
				Case Else
					Return
			End Select
		Wend

GetSuffix:
		If curChar_ = ll_ Then Return
		fb = @p[curChar_]
		While fb = @p[curChar_]
			Select Case Const @p[curChar_]
					' !#%?&$
				Case 33, 35, 37, 63, 38, 36
					Incr curChar_
				Case Else
					Return
			End Select
		Wend

GetIdentifier:
		Select Case Const @p[curChar_]
				' A-Z, a-z
			Case 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90 _
						, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122
				Incr curChar_
			Case Else
				Return
		End Select
		While 1
			Select Case Const @p[curChar_]
					' A-Z, a-z, 0-9, _
				Case 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90 _
							, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122 _
							, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 95
					Incr curChar_
				Case Else
					Return
			End Select
		Wend
		#Debug Code On
	End Method
End Class
