'===================================================================================
' PBWin or PBCC
' This project will standardize the export aliases in PbSlice. i.e. ensure that any
' routines that are exported have an alias that matches the case of the routine.
' It also generates a first pass at an include file for the DLL.  Once the include file
' has been created, remove any constants, the type with a comment it's for internal use,
' and all interfaces.  The file name of the include file is PbSlice200DLL.inc.
'
' This is also an example of how to include the PbSlice source code in your project.
' Just ensure step 1 is done once with each compiling environment you use, ensure
' step 2 has been done once on the computer.

' To compile ensure:
' 1) The IDE/Compiler search path includes the full path to the src sub directory
' 2) Ensure src\PbSlice.inc has the path you placed the source in.  Replace ALL
'    c:\Prj\Pb\PbSlice2\Src with the directory you're copy of the source is.
' 3) Search for '==  Ensure the correct section is uncommented.  You can uncomment both
'    it will simply remove the aliases and then add them back.
' 4) Set $SRC_PATH to the location the PbSlice project is in.
' 5) Set $DST_PATH to where you want to save changes.
'
' $SRC_PATH and $DST_PATH can be the same, this just illustrated an additional
' feature (copying a project to a new directory)
'===================================================================================

#Compile Exe
#Compiler PBWin 10, PBCC 6
#Dim All
#Debug Display On
#Debug Error On
#Optimize Code Off
#Tools On

$SRC_PATH = "f:\Prj\Pb\PbSlice2\"
$DST_PATH = "f:\Prj\Pb\PbSlice2Tmp\"

' Including PBSlice source directly (PbSlice200DLL.inc may not exist or be correct).
#Include "PBSlice2.inc"

Function PBMain () As Long
	Local a As String

	Call pbsKeywordLoad( "pbwin10" ):                         ' Load built-in keywords
	Call pbsProjectSetIncludePaths( $SRC_PATH + "src" )       ' Need to ensure we can load the project

	pbsProjectLoad( $SRC_PATH + "PbSlice200.bas" ):           ' Load the project
	pbsProjectSetBasePath( $DST_PATH ):                       ' Save changes to tmp directory

	pbsProjectDeclareScan():                                  ' Scan for declarations

	'====================================================
	' Uncomment next line to remove aliases
	'pbsProjectRemoveAliases():                                ' Remove aliases
	'----------------------------------------------------
	' Uncomment next two lines to standardize aliases
	a = pbsProjectFindExports():                              ' Get list of any routines in #EXPORT statements
	pbsProjectStandardizeExportAliases(a):                    ' Fix-up aliases in source file for any routines with Export attributes
	'----------------------------------------------------
	' Uncoment next line to add debugging aliases
	'pbsProjectDebugAliases():                                 ' Set debugging aliases
	'====================================================

	pbsProjectSave():                                         ' Save project with standardized aliases

	a = pbsProjectBuildDllInclude()                           ' Generate include file for DLL
	pbsSnippetNew("dllinc"):                                  ' Initialize a new snippet
	pbsSnippetSetText( a ):                                   ' Make snippet the dll text
	pbsSnippetSetFilename( $DST_PATH + "PbSlice200DLL.inc" ): ' Set the path for the snippet
	pbsSnippetSave():                                         ' Save DLL include file
	Clipboard Reset
	Clipboard Set Text a:                                     ' Place a copy on the clipboard.
#If %Def( %PB_CC32 )
	Con.Print "DLL include is also on clipboard"
	Con.Print
	Con.Print "Press ENTER to exit"
	Con.Line.Input a
#ElseIf %Def( %PB_Win32 )
	MsgBox Build$( "DLL include is also on clipboard" )
#EndIf
End Function
